/*
 * YouGrabber
 *
 * A lightweight, ncurses-based, multi-threaded command line YouTube.com
 * video downloader with proxy support, made from scratch in ANSI C.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "common.h"

static const char __CONFIG_FILE[] = {".yg.conf"};

/* key file groups/keys */
static const char __GROUP_PROXY[] = {"proxy"};
static const char __GROUP_YOUTUBE[] = {"youtube"};
static const char __KEY_PROXY_HOST[] = {"host"};
static const char __KEY_PROXY_PORT[] = {"port"};
static const char __KEY_PROXY_TYPE[] = {"type"};
static const char __KEY_PROXY_USR[] = {"username"};
static const char __KEY_PROXY_PWD[] = {"password"};
static const char __KEY_YOUTUBE_USR[] = {"username"};
static const char __KEY_YOUTUBE_PWD[] = {"password"};

struct __config
{
  char file[64];

  struct
  {
    char host[256];
    long port;
    long type;
    char username[32];
    char password[32];
  } proxy;

  struct
  {
    char username[32];
    char password[32];
    unsigned short login_status;
  } youtube;
};

/* set configuration and return pointer */
C *config_new(void)
{
  C *c_ptr = NULL;
  GKeyFile *kf = NULL;

  if (!(c_ptr = (C*) malloc(sizeof(*c_ptr))))
    return NULL;

  /* file path */
  g_snprintf(c_ptr->file, sizeof(c_ptr->file), "%s/%s",
    g_get_home_dir(), __CONFIG_FILE);

  /* proxy */
  g_strlcpy(c_ptr->proxy.host, "", sizeof(c_ptr->proxy.host));
  c_ptr->proxy.port = 8080;
  c_ptr->proxy.type = CURLPROXY_HTTP;
  g_strlcpy(c_ptr->proxy.username, "", sizeof(c_ptr->proxy.username));
  g_strlcpy(c_ptr->proxy.password, "", sizeof(c_ptr->proxy.password));

  /* youtube */
  g_strlcpy(c_ptr->youtube.username, "", sizeof(c_ptr->youtube.username));
  g_strlcpy(c_ptr->youtube.password, "", sizeof(c_ptr->youtube.password));
  c_ptr->youtube.login_status = LOGIN_WAITING;

  kf = g_key_file_new();

  /* open configuration file */
  if (!g_key_file_load_from_file(kf, c_ptr->file, G_KEY_FILE_NONE, NULL))
  {
    g_strlcpy(c_ptr->file, "", sizeof(c_ptr->file));
    g_key_file_free(kf);

    return c_ptr;
  }

  /* proxy group */
  if (g_key_file_has_group(kf, __GROUP_PROXY))
  {
    if (g_key_file_has_key(kf, __GROUP_PROXY, __KEY_PROXY_HOST, NULL))
      g_strlcpy(c_ptr->proxy.host, g_key_file_get_value(kf, __GROUP_PROXY,
        __KEY_PROXY_HOST, NULL), sizeof(c_ptr->proxy.host));

    if (g_key_file_has_key(kf, __GROUP_PROXY, __KEY_PROXY_PORT, NULL))
      c_ptr->proxy.port = atol(g_key_file_get_value(kf, __GROUP_PROXY,
        __KEY_PROXY_PORT, NULL));

    if (g_key_file_has_key(kf, __GROUP_PROXY, __KEY_PROXY_TYPE, NULL))
    {
      char *type_str = g_key_file_get_value(kf, __GROUP_PROXY,
        __KEY_PROXY_TYPE, NULL);

      if (!g_ascii_strcasecmp(type_str, "HTTP"))
        c_ptr->proxy.type = CURLPROXY_HTTP;

      if (!g_ascii_strcasecmp(type_str, "SOCKS4"))
        c_ptr->proxy.type = CURLPROXY_SOCKS4;

      if (!g_ascii_strcasecmp(type_str, "SOCKS5"))
        c_ptr->proxy.type = CURLPROXY_SOCKS5;
    }

    if (g_key_file_has_key(kf, __GROUP_PROXY, __KEY_PROXY_USR, NULL))
      g_strlcpy(c_ptr->proxy.username, g_key_file_get_value(kf, __GROUP_PROXY,
        __KEY_PROXY_USR, NULL), sizeof(c_ptr->proxy.username));

    if (g_key_file_has_key(kf, __GROUP_PROXY, __KEY_PROXY_PWD, NULL))
      g_strlcpy(c_ptr->proxy.password, g_key_file_get_value(kf, __GROUP_PROXY,
        __KEY_PROXY_PWD, NULL), sizeof(c_ptr->proxy.password));
  }

  /* youtube group */
  if (g_key_file_has_group(kf, __GROUP_YOUTUBE))
  {
    if (g_key_file_has_key(kf, __GROUP_YOUTUBE, __KEY_YOUTUBE_USR, NULL))
      g_strlcpy(c_ptr->youtube.username, g_key_file_get_value(kf,
        __GROUP_YOUTUBE, __KEY_YOUTUBE_USR, NULL),
        sizeof(c_ptr->youtube.username));

    if (g_key_file_has_key(kf, __GROUP_YOUTUBE, __KEY_YOUTUBE_PWD, NULL))
      g_strlcpy(c_ptr->youtube.password, g_key_file_get_value(kf,
        __GROUP_YOUTUBE, __KEY_YOUTUBE_PWD, NULL),
        sizeof(c_ptr->youtube.password));
  }

  g_key_file_free(kf);

  return c_ptr;
}

/* free configuration resources */
void config_free(C *c_ptr)
{
  if (g_file_test(OREO, G_FILE_TEST_EXISTS))
    remove(OREO);

  free(c_ptr);
}

/* get configuration file path */
char *config_get_file_path(C *c_ptr)
{
  return c_ptr->file;
}

/* get configuration proxy host */
char *config_get_proxy_host(C *c_ptr)
{
  return c_ptr->proxy.host;
}

/* get configuration proxy port */
long config_get_proxy_port(C *c_ptr)
{
  return c_ptr->proxy.port;
}

/* get configuration proxy type */
long config_get_proxy_type(C *c_ptr)
{
  return c_ptr->proxy.type;
}

/* get configuration proxy username */
char *config_get_proxy_username(C *c_ptr)
{
  return c_ptr->proxy.username;
}

/* get configuration proxy password */
char *config_get_proxy_password(C *c_ptr)
{
  return c_ptr->proxy.password;
}

/* get configuration youtube username */
char *config_get_youtube_username(C *c_ptr)
{
  return c_ptr->youtube.username;
}

/* get configuration youtube password */
char *config_get_youtube_password(C *c_ptr)
{
  return c_ptr->youtube.password;
}

/* get configuration youtube login status */
int config_get_youtube_login_status(C *c_ptr)
{
  return c_ptr->youtube.login_status;
}

/* set configuration youtube login status */
void config_set_youtube_login_status(C *c_ptr, int login_status)
{
  c_ptr->youtube.login_status = login_status;
}
