/*
 * YouGrabber
 *
 * A lightweight, ncurses-based, multi-threaded command line YouTube.com
 * video downloader with proxy support, made from scratch in ANSI C.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "common.h"

/* file stuff */
static const char __LOGIN_DATA[] = {"login.dat"};
static char __FILE_NAME_TMP[] = {"VIDEODATA"};
static char __FILE_EXT_TMP[] = {"part"};
static char __FILE_EXT_FLV[] = {"flv"};

/* strcut() match strings and delimiters */
static const char __FLAGGED_URL_MATCH[] = {"verify_age?"};

static const char __NEXT_URL_MATCH[] = {"name=\"next_url\""};
static char __NEXT_URL_DELIM_1[] = {"value=\""};
static char __NEXT_URL_DELIM_2[] = {"\">"};

static const char __TITLE_MATCH[] = {"<meta name=\"title\""};
static char __TITLE_DELIM_1[] = {"content=\""};
static char __TITLE_DELIM_2[] = {"\">"};

static const char __URL_MATCH[] = {"swfArgs"};
static char __URL_DELIM_ID_1[] = {"video_id:'"};
static char __URL_DELIM_ID_2[] = {"',"};
static char __URL_DELIM_T_1[] = {"t:'"};
static char __URL_DELIM_T_2[] = {"',"};

/* youtube URLs */
static const char __YOUTUBE_URL_LOGIN[] =
  {"http://www.youtube.com/login"};
static const char __YOUTUBE_URL_VIDEO[] =
  {"http://www.youtube.com/get_video?video_id="};

/* video error match strings */
static const char __YOUTUBE_ERROR_UNAVAILABLE[] =
  {"The video you have requested is not available"};
static const char __YOUTUBE_ERROR_REMOVED[] =
  {"This video has been removed due to terms of use violation"};

struct __download
{
  char network_error[CURL_ERROR_SIZE];
  unsigned short status;
  CURL *easy_handle;
  double time;
  double speed;
  C *c_ptr;

  struct
  {
    char name[96];
    double size_now;
    double size_total;
  } file;

  struct
  {
    char title[64];
    char url[256];
  } video;

  pthread_t thread;
};

/* update download data (cURL progress callback function) */
static int __download_update(void *clientp, double dltotal, double dlnow,
  double ultotal, double ulnow)
{
  char *url_ptr = NULL;
  D *d_ptr = NULL;

  (void) ultotal;
  (void) ulnow;

  d_ptr = (D*) clientp;

  d_ptr->file.size_now = dlnow;
  d_ptr->file.size_total = dltotal;

  curl_easy_getinfo(d_ptr->easy_handle, CURLINFO_EFFECTIVE_URL, &url_ptr);
  g_strlcpy(d_ptr->video.url, url_ptr, sizeof(d_ptr->video.url));

  curl_easy_getinfo(d_ptr->easy_handle, CURLINFO_TOTAL_TIME, &d_ptr->time);

  curl_easy_getinfo(d_ptr->easy_handle, CURLINFO_SPEED_DOWNLOAD,
    &d_ptr->speed);

  return 0;
}

/* grab the video */
static int __download_grab(D *d_ptr)
{
  FILE *stream = NULL;
  unsigned v_title = 0;
  unsigned v_url = 0;
  char v_next_url[32];
  char buf[256];
  char aux[256];

  /* generate file name */
  if (!file_name_gen(d_ptr->file.name, __FILE_NAME_TMP, __FILE_EXT_TMP))
    d_ptr->status = DOWNLOAD_ERROR_IO;

  /* open file */
  if (!(stream = fopen(d_ptr->file.name, "w+")))
    d_ptr->status = DOWNLOAD_ERROR_IO;

  /* download step 1 */
  if (d_ptr->status == DOWNLOAD_OK_DOWNLOADING)
  {
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_ERRORBUFFER,
      d_ptr->network_error);

    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_URL, d_ptr->video.url);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_WRITEDATA, stream);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_AUTOREFERER, 1);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_FOLLOWLOCATION, 1);

    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_NOPROGRESS, 0);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROGRESSDATA, d_ptr);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROGRESSFUNCTION,
      __download_update);

    /* use cookies if we're logged */
    if (config_get_youtube_login_status(d_ptr->c_ptr) == LOGIN_OK)
    {
      curl_easy_setopt(d_ptr->easy_handle, CURLOPT_COOKIEFILE, OREO);
      curl_easy_setopt(d_ptr->easy_handle, CURLOPT_COOKIEJAR, OREO);
    }

    /* set proxy */
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROXY,
      config_get_proxy_host(d_ptr->c_ptr));

    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROXYPORT,
      config_get_proxy_port(d_ptr->c_ptr));

    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROXYTYPE,
      config_get_proxy_type(d_ptr->c_ptr));

    g_snprintf(buf, sizeof(buf), "%s:%s",
      config_get_proxy_username(d_ptr->c_ptr),
      config_get_proxy_password(d_ptr->c_ptr));

    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROXYUSERPWD, buf);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_PROXYAUTH, CURLAUTH_ANYSAFE);

    if (curl_easy_perform(d_ptr->easy_handle) != CURLE_OK)
      d_ptr->status = DOWNLOAD_ERROR_NETWORK;
  }

  /* only post confirmation on flagged videos if we're logged */
  if (strstr(d_ptr->video.url, __FLAGGED_URL_MATCH)
    && config_get_youtube_login_status(d_ptr->c_ptr) == LOGIN_OK)
  {
    fflush(stream);
    rewind(stream);

    while (fgets(buf, sizeof(buf), stream))
    {
      if (feof(stream))
        break;

      /* next URL */
      if (strstr(buf, __NEXT_URL_MATCH))
        g_strlcpy(v_next_url, strcut(buf, __NEXT_URL_DELIM_1,
          __NEXT_URL_DELIM_2), sizeof(v_next_url));
    }

    /* post */
    g_snprintf(aux, sizeof(aux),
      "next_url=%s&action_confirm=Confirm+Birth+Date", v_next_url);

    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_POST, 1);
    curl_easy_setopt(d_ptr->easy_handle, CURLOPT_POSTFIELDS, aux);

    if (curl_easy_perform(d_ptr->easy_handle) != CURLE_OK)
      d_ptr->status = DOWNLOAD_ERROR_NETWORK;
  }

  /* download step 2 */
  if (d_ptr->status == DOWNLOAD_OK_DOWNLOADING)
  {
    fflush(stream);
    rewind(stream);

    while (fgets(buf, sizeof(buf), stream) || (!v_title && !v_url)
      || d_ptr->status != DOWNLOAD_OK_DOWNLOADING)
    {
      if (feof(stream))
        break;

      /* video unavailable */
      if (strstr(buf, __YOUTUBE_ERROR_UNAVAILABLE))
        d_ptr->status = DOWNLOAD_ERROR_VIDEO_UNAVAILABLE;

      /* video removed */
      if (strstr(buf, __YOUTUBE_ERROR_REMOVED))
        d_ptr->status = DOWNLOAD_ERROR_VIDEO_REMOVED;

      /* video title */
      if (strstr(buf, __TITLE_MATCH))
      {
        g_strlcpy(d_ptr->video.title, strcut(buf, __TITLE_DELIM_1,
          __TITLE_DELIM_2), sizeof(d_ptr->video.title));

        file_name_gen(aux, d_ptr->video.title, __FILE_EXT_FLV);

        v_title = 1;
      }

      /* video URL fragments */
      if (strstr(buf, __URL_MATCH))
      {
        char v_id[strlen(buf)];
        char v_t[strlen(buf)];

        g_strlcpy(v_id, buf, sizeof(v_id));
        g_strlcpy(v_t, buf, sizeof(v_t));

        strcut(v_id, __URL_DELIM_ID_1, __URL_DELIM_ID_2);
        strcut(v_t, __URL_DELIM_T_1, __URL_DELIM_T_2);

        g_snprintf(buf, sizeof(buf), "%s%s&t=%s", __YOUTUBE_URL_VIDEO,
          v_id, v_t);

        g_strlcpy(d_ptr->video.url, buf, sizeof(d_ptr->video.url));

        v_url = 1;
      }
    }
  }

  /* download step 3 */
  if (d_ptr->status == DOWNLOAD_OK_DOWNLOADING)
  {
    /* get video */
    if (v_title && v_url)
    {
      fflush(stream);
      rewind(stream);

      curl_easy_setopt(d_ptr->easy_handle, CURLOPT_URL, d_ptr->video.url);

      if (curl_easy_perform(d_ptr->easy_handle) != CURLE_OK)
        d_ptr->status = DOWNLOAD_ERROR_NETWORK;
    }
    else
      d_ptr->status = DOWNLOAD_ERROR_VIDEO_UNAVAILABLE;
  }

  /* download step 4 */
  if (d_ptr->status == DOWNLOAD_OK_DOWNLOADING)
  {
    fflush(stream);
    fclose(stream);

    if (!rename(d_ptr->file.name, aux))
      g_strlcpy(d_ptr->file.name, aux, sizeof(d_ptr->file.name));

    d_ptr->status = DOWNLOAD_OK_FINISHED;
  }
  else
  {
    fclose(stream);
    remove(d_ptr->file.name);
  }

  dip_rem();

  return d_ptr->status;
}

/* create a new download and return pointer */
D *download_new(C *c_ptr, char *url)
{
  D *d_ptr = NULL;

  if (!(d_ptr = (D *) malloc(sizeof(D))))
    return NULL;

  if (!(d_ptr->easy_handle = curl_easy_init()))
  {
    free(d_ptr);

    return NULL;
  }

  d_ptr->c_ptr = c_ptr;
  d_ptr->status = DOWNLOAD_OK_DOWNLOADING;

  g_strlcpy(d_ptr->file.name, "", sizeof(d_ptr->file.name));
  d_ptr->file.size_now = 0.0;
  d_ptr->file.size_total = 0.0;

  g_strlcpy(d_ptr->video.title, url, sizeof(d_ptr->video.title));
  g_strlcpy(d_ptr->video.url, url, sizeof(d_ptr->video.url));

  if (pthread_create(&d_ptr->thread, NULL, (void *) __download_grab, d_ptr))
  {
    curl_easy_cleanup(d_ptr->easy_handle);
    free(d_ptr);

    return NULL;
  }

  return d_ptr;
}

/* free download resources (GFunc() type function) */
void download_free(gpointer data, gpointer user_data)
{
  D *d_ptr = NULL;

  (void) user_data;

  d_ptr = (D*) data;

  pthread_join(d_ptr->thread, NULL);
  curl_easy_cleanup(d_ptr->easy_handle);
  free(d_ptr);
}

/* login into youtube and get cookie */
int download_cookie(C *c_ptr)
{
  unsigned logged = 0;
  char post[128];
  char buf[256];
  char *url_ptr;
  FILE *stream = NULL;
  CURL *easy_handle = NULL;

  if (strlen(config_get_youtube_username(c_ptr)) < 1
    || strlen(config_get_youtube_password(c_ptr)) < 1)
    return LOGIN_SKIPPED;

  if (!(stream = fopen(__LOGIN_DATA, "w+")))
    return LOGIN_FAILED;

  if ((easy_handle = curl_easy_init()))
  {
    curl_easy_setopt(easy_handle, CURLOPT_URL, __YOUTUBE_URL_LOGIN);
    curl_easy_setopt(easy_handle, CURLOPT_WRITEDATA, stream);

    curl_easy_setopt(easy_handle, CURLOPT_AUTOREFERER, 1);
    curl_easy_setopt(easy_handle, CURLOPT_FOLLOWLOCATION, 1);
    curl_easy_setopt(easy_handle, CURLOPT_NOPROGRESS, 1);

    curl_easy_setopt(easy_handle, CURLOPT_COOKIEFILE, OREO);
    curl_easy_setopt(easy_handle, CURLOPT_COOKIEJAR, OREO);

    curl_easy_setopt(easy_handle, CURLOPT_PROXY,
      config_get_proxy_host(c_ptr));

    curl_easy_setopt(easy_handle, CURLOPT_PROXYPORT,
      config_get_proxy_port(c_ptr));

    curl_easy_setopt(easy_handle, CURLOPT_PROXYTYPE,
      config_get_proxy_type(c_ptr));

    g_snprintf(buf, sizeof(buf), "%s:%s", config_get_proxy_username(c_ptr),
      config_get_proxy_password(c_ptr));

    curl_easy_setopt(easy_handle, CURLOPT_PROXYUSERPWD, buf);
    curl_easy_setopt(easy_handle, CURLOPT_PROXYAUTH, CURLAUTH_ANYSAFE);

    /* post */
    g_snprintf(post, sizeof(post),
      "current_form=loginForm&username=%s&password=%s&action_login=Log+In",
      config_get_youtube_username(c_ptr), config_get_youtube_password(c_ptr));

    curl_easy_setopt(easy_handle, CURLOPT_POST, 1);
    curl_easy_setopt(easy_handle, CURLOPT_POSTFIELDS, post);

    /* check if we're logged */
    if (curl_easy_perform(easy_handle) == CURLE_OK)
    {
      curl_easy_getinfo(easy_handle, CURLINFO_EFFECTIVE_URL, &url_ptr);

      logged = !strstr(url_ptr, __YOUTUBE_URL_LOGIN);
    }

    curl_easy_cleanup(easy_handle);
  }

  fflush(stream);
  fclose(stream);
  remove(__LOGIN_DATA);

  return (logged) ? LOGIN_OK : LOGIN_FAILED;
}

/* get download status */
int download_get_status(D *d_ptr)
{
  return d_ptr->status;
}

/* get download network error */
char *download_get_network_error(D *d_ptr)
{
  return d_ptr->network_error;
}

/* get download speed */
double download_get_speed(D *d_ptr)
{
  return d_ptr->speed;
}

/* get download time */
double download_get_time(D *d_ptr)
{
  return d_ptr->time;
}

/* get download file name */
char *download_get_file_name(D *d_ptr)
{
  return d_ptr->file.name;
}

/* get download current file size */
double download_get_file_size_now(D *d_ptr)
{
  return d_ptr->file.size_now;
}

/* get download total file size */
double download_get_file_size_total(D *d_ptr)
{
  return d_ptr->file.size_total;
}

/* get download video title */
char *download_get_video_title(D *d_ptr)
{
  return d_ptr->video.title;
}

/* get download video URL */
char *download_get_video_url(D *d_ptr)
{
  return d_ptr->video.url;
}
