/*
 * YouGrabber
 *
 * A lightweight, ncurses-based, multi-threaded command line YouTube.com
 * video downloader with proxy support, made from scratch in ANSI C.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "common.h"

static const double __MEGABYTE = 1024.0*1024.0;
static const double __KILOBYTE = 1024.0;
static const double __MINUTE = 59.0;
static const double __PERCENT = 100.0;

/* interface current and minimum sizes */
static unsigned short __LINES = 0;
static unsigned short __COLS = 0;
static const unsigned short __MIN_LINES = 20;
static const unsigned short __MIN_COLS = 80;

struct __interface
{
  C *c_ptr;
  GList *d_list;
  WINDOW *window;
  D *d_ptr;
};

/* start ncurses */
static void __interface_nc_start(void)
{
  initscr();
  cbreak();
  noecho();
  curs_set(0);
  nodelay(stdscr, TRUE); /* very handy for the loop */
  keypad(stdscr, TRUE);
}

/* stop ncurses */
static void __interface_nc_stop(void)
{
  clear();
  refresh();
  endwin();
}

/* set the interface size variables */
static void __interface_set_size(I *i_ptr)
{
  getmaxyx(i_ptr->window, __LINES, __COLS);

  __LINES = (__LINES < __MIN_LINES) ? __MIN_LINES : __LINES;
  __COLS = (__COLS < __MIN_COLS) ? __MIN_COLS : __COLS;
}

/* update the interface */
static void __interface_update(I *i_ptr)
{
  char *file_path = NULL;
  char *proxy_host = NULL;
  char *url_ptr = NULL;
  double dlnow = 0.0;
  double dltotal = 0.0;
  double time = 0.0;

  char *status_youtube[] =
  {
    "Failed (flagged downloads will be unavailable)",
    "Skipped (flagged downloads will be unavailable)",
    "Waiting...",
    "Logged"
  };

  char *status_download[] =
  {
    "Video is unavailable",
    "Video has been removed",
    "I/O error",
    download_get_network_error(i_ptr->d_ptr),
    "Downloading video data...",
    "Download finished!"
  };

  file_path = config_get_file_path(i_ptr->c_ptr);
  proxy_host = config_get_proxy_host(i_ptr->c_ptr);
  dlnow = download_get_file_size_now(i_ptr->d_ptr);
  dltotal = download_get_file_size_total(i_ptr->d_ptr);
  time = download_get_time(i_ptr->d_ptr);
  url_ptr = download_get_video_url(i_ptr->d_ptr);

  /* top */
  werase(i_ptr->window);

  mvwprintw(i_ptr->window, 0, 0, "%s v%s - Copyright (C) %s <%s>",
    YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL);

  mvwhline(i_ptr->window, 1, 0, ACS_HLINE, __COLS);

  mvwprintw(i_ptr->window, 3, 1, "YOUTUBE LOGIN STATE: %s",
    status_youtube[config_get_youtube_login_status(i_ptr->c_ptr)]);

  mvwprintw(i_ptr->window, 4, 1, "SETTINGS: %s",
    (strlen(file_path) > 0) ? file_path : "Default");

  mvwprintw(i_ptr->window, 5, 1,  "PROXY: %s",
    (strlen(proxy_host) > 0) ? proxy_host : "None");

  mvwprintw(i_ptr->window, 7, 1, "DOWNLOADS REMAINING: %.3d", dip_get());

  /* download */
  mvwhline(i_ptr->window, __LINES-11, 0, ACS_HLINE, __COLS);

  mvwprintw(i_ptr->window, __LINES-11, 2, " DOWNLOAD %.3d/%.3d ",
    g_list_index(i_ptr->d_list, i_ptr->d_ptr)+1,
    g_list_length(i_ptr->d_list));

  mvwprintw(i_ptr->window, __LINES-10, 0, "TITLE: %s",
    download_get_video_title(i_ptr->d_ptr));

  mvwprintw(i_ptr->window, __LINES-9, 0, "FILE: %s",
    download_get_file_name(i_ptr->d_ptr));

  mvwprintw(i_ptr->window, __LINES-8, 0, "STATUS: %s",
    status_download[download_get_status(i_ptr->d_ptr)]);

  mvwprintw(i_ptr->window, __LINES-7, 0,
    "DOWNLOADED: %.2f%% (%.2f of %.2f MB)\n",
    (dlnow*__PERCENT)/dltotal, dlnow/__MEGABYTE, dltotal/__MEGABYTE);

  mvwprintw(i_ptr->window, __LINES-6, 0, "SPEED: %.2f KB/sec",
    download_get_speed(i_ptr->d_ptr)/__KILOBYTE);

  mvwprintw(i_ptr->window, __LINES-5, 0, "ELAPSED: %d min %.f sec",
    (int) (time/__MINUTE), fmod(time, __MINUTE));

  mvwprintw(i_ptr->window, __LINES-4, 0,
    (strlen(url_ptr) > 128) ? "URL: %.128s (...)" : "URL: %s", url_ptr);

  /* bottom */
  mvwhline(i_ptr->window, __LINES-2, 0, ACS_HLINE, __COLS);

  mvwprintw(i_ptr->window, __LINES-1, 0,
    "LEFT/RIGHT ARROW KEYS: browse between downloads - %s",
    (dip_get() > 0) ? "Wait for downloads to finish" : "Press ESC to exit");

  wrefresh(i_ptr->window);
}

/* create interface and return pointer */
I *interface_new(C *c_ptr, GList *d_list)
{
  I *i_ptr;

  i_ptr = NULL;

  if (!(i_ptr = (I*) malloc(sizeof(I))))
    return NULL;

  __interface_nc_start();

  i_ptr->c_ptr = c_ptr;
  i_ptr->d_list = d_list;
  i_ptr->window = stdscr; /* we only use the default window */
  i_ptr->d_ptr = NULL;

  return i_ptr;
}

/* free interface resources */
void interface_free(I *i_ptr)
{
  __interface_nc_stop();

  g_list_foreach(i_ptr->d_list, download_free, NULL);
  g_list_free(i_ptr->d_list);

  config_free(i_ptr->c_ptr);

  free(i_ptr);
}

/* interface event processing loop */
void interface_main_loop(I *i_ptr)
{
  unsigned short quit = 0;
  GList *node_current = NULL;
  GList *node_next = NULL;

  node_current = g_list_first(i_ptr->d_list);

  __interface_set_size(i_ptr);

  while(!quit)
  {
    node_current = (!node_next) ? node_current : node_next;
    i_ptr->d_ptr = node_current->data;

    __interface_update(i_ptr);

    switch (getch())
    {
      case ERR:
        __interface_update(i_ptr);
      break;

      case KEY_RESIZE:
        __interface_set_size(i_ptr);
        __interface_update(i_ptr);
      break;

      case KEY_LEFT:
        node_next = g_list_previous(node_current);
      break;

      case KEY_RIGHT:
        node_next = g_list_next(node_current);
      break;

      case 27: /* ESC */
        quit = !dip_get();
      break;
    }
  }
}
